"""Template modules for dataset formatting."""
from .gsm8k_template import GSM8KTemplate
from .openmathinstruct_template import OpenMathInstructTemplate

__all__ = ['GSM8KTemplate', 'OpenMathInstructTemplate']
