"""
Setup script for FAI-RL package.
This file exists for backwards compatibility.
The main configuration is in pyproject.toml.
"""

from setuptools import setup

if __name__ == "__main__":
    setup()

